"""The ch commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CH<x>:BANdwidth {TWEnty|FULl|<NR3>}
    - CH<x>:BANdwidth?
    - CH<x>:COUPling {AC|DC|GND}
    - CH<x>:COUPling?
    - CH<x>:DESKew <NR3>
    - CH<x>:DESKew?
    - CH<x>:IMPedance
    - CH<x>:IMPedance?
    - CH<x>:INVert {ON|OFF}
    - CH<x>:INVert?
    - CH<x>:LABel <Qstring>
    - CH<x>:LABel?
    - CH<x>:OFFSet <NR3>
    - CH<x>:OFFSet?
    - CH<x>:POSition <NR3>
    - CH<x>:POSition?
    - CH<x>:PRObe:AUTOZero EXECute
    - CH<x>:PRObe:COMMAND <QString>, <QString>
    - CH<x>:PRObe:DEGAUss EXECute
    - CH<x>:PRObe:DEGAUss:STATE?
    - CH<x>:PRObe:FORCEDRange <NR3>
    - CH<x>:PRObe:FORCEDRange?
    - CH<x>:PRObe:GAIN <NR3>
    - CH<x>:PRObe:GAIN?
    - CH<x>:PRObe:ID:SERnumber?
    - CH<x>:PRObe:ID:TYPE?
    - CH<x>:PRObe:ID?
    - CH<x>:PRObe:RESistance?
    - CH<x>:PRObe:SIGnal {BYPass|PASS}
    - CH<x>:PRObe:SIGnal?
    - CH<x>:PRObe:UNIts?
    - CH<x>:PRObe?
    - CH<x>:SCAle <NR3>
    - CH<x>:SCAle?
    - CH<x>:TERmination
    - CH<x>:TERmination?
    - CH<x>:VOLts <NR3>
    - CH<x>:VOLts?
    - CH<x>:YUNits <QString>
    - CH<x>:YUNits?
    - CH<x>?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments, ValidatedChannel

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class ChannelYunits(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:YUNits`` command.

    Description:
        - This command specifies the vertical units for the channel specified by <x>, where x is the
          channel number. String arguments are case insensitive and any unsupported units will
          generate an error. Supported units are: %, /Hz, A, A/A, A/V, A/W, A/dB, A/s, AA, AW, AdB,
          As, B, Hz, IRE, S/s, V, V/A, V/V, V/W, V/dB, V/s, VV, VW, VdB, volts, Vs, W, W/A, W/V,
          W/W, W/dB, W/s, WA, WV, WW, WdB, Ws, dB, dB/A, dB/V, dB/W, dB/dB, dBA, dBV, dBW, dBdB,
          day, degrees, div, hr, min, ohms, percent, s The vertical units affect the 'Probe Type'
          that is shown in the 'Probe Setup' menu: Setting ``CH<x>:YUNits`` to 'V' causes the probe
          type to be displayed as 'Voltage'. When ``CH1:AMSVIAVOLTs:ENAble`` is set to OFF, setting
          ``CH<x>:YUNits`` to 'A' causes the probe type to be displayed as 'Current'. Setting
          ``CH<x>:YUNits`` to anything else causes the probe type not to be displayed (neither
          'Voltage' nor 'Current' are highlighted).

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:YUNits?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:YUNits?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:YUNits value`` command.

    SCPI Syntax:
        ```
        - CH<x>:YUNits <QString>
        - CH<x>:YUNits?
        ```

    Info:
        - ``QString`` is a string of text surrounded by quotes, specifying the supported units.
    """

    _WRAP_ARG_WITH_QUOTES = True


class ChannelVolts(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:VOLts`` command.

    Description:
        - This command specifies the vertical sensitivity for channel <x>, where x is the channel
          number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:VOLts?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:VOLts?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:VOLts value`` command.

    SCPI Syntax:
        ```
        - CH<x>:VOLts <NR3>
        - CH<x>:VOLts?
        ```

    Info:
        - ``<NR3>`` is the vertical sensitivity, in volts.
    """


class ChannelTermination(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``CH<x>:TERmination`` command.

    Description:
        - Sets the connected-disconnected status of a 50 Ω resistor, which may be connected between
          the specified channel's coupled input and oscilloscope ground. The channel is specified by
          <x>. There is also a corresponding query that requests the termination parameter and
          translates this enumeration into one of the two float values. This command is maintained
          for compatibility.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:TERmination?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:TERmination?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write()`` method will send the ``CH<x>:TERmination`` command.

    SCPI Syntax:
        ```
        - CH<x>:TERmination
        - CH<x>:TERmination?
        ```
    """


class ChannelScale(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:SCAle`` command.

    Description:
        - This command sets or returns the vertical scale for the specified analog channel. The
          channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:SCAle?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:SCAle value`` command.

    SCPI Syntax:
        ```
        - CH<x>:SCAle <NR3>
        - CH<x>:SCAle?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the vertical scale for the specified analog channel.
    """


class ChannelProbeUnits(SCPICmdRead):
    """The ``CH<x>:PRObe:UNIts`` command.

    Description:
        - Returns a string describing the units of measure for the probe attached to channel <x>,
          where x is the channel number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:UNIts?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:UNIts?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:UNIts?
        ```
    """


class ChannelProbeSignal(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PRObe:SIGnal`` command.

    Description:
        - This command specifies the input bypass setting of a TekVPI probe attached to channel <x>,
          where x is the channel number. The probe must support input bypass, for example TCP0001.
          This command is ignored if sent to an unsupported probe.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:SIGnal?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:SIGnal?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:SIGnal value`` command.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:SIGnal {BYPass|PASS}
        - CH<x>:PRObe:SIGnal?
        ```

    Info:
        - ``BYPass`` sets the probe to Bypass mode.
        - ``PASS`` sets the probe to Pass mode.
    """


class ChannelProbeResistance(SCPICmdRead):
    """The ``CH<x>:PRObe:RESistance`` command.

    Description:
        - Returns the resistance factor of the probe attached to channel <x>, where x is the channel
          number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:RESistance?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:RESistance?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:RESistance?
        ```
    """


class ChannelProbeIdType(SCPICmdRead):
    """The ``CH<x>:PRObe:ID:TYPE`` command.

    Description:
        - Returns the type of probe attached to the channel specified by <x>, where x is the channel
          number. Level 2 (or higher) probes supply their exact product nomenclature; for Level 0 or
          1 probes, a generic 'No Probe Detected message is returned.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:ID:TYPE?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:ID:TYPE?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:ID:TYPE?
        ```
    """


class ChannelProbeIdSernumber(SCPICmdRead):
    """The ``CH<x>:PRObe:ID:SERnumber`` command.

    Description:
        - Returns the serial number of the probe attached to channel <x>, where x is the channel
          number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:ID:SERnumber?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:ID:SERnumber?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:ID:SERnumber?
        ```
    """


class ChannelProbeId(SCPICmdRead):
    """The ``CH<x>:PRObe:ID`` command.

    Description:
        - Returns the type and serial number of the probe attached to channel <x>, where x is the
          channel number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:ID?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:ID?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:ID?
        ```

    Properties:
        - ``.sernumber``: The ``CH<x>:PRObe:ID:SERnumber`` command.
        - ``.type``: The ``CH<x>:PRObe:ID:TYPE`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._sernumber = ChannelProbeIdSernumber(device, f"{self._cmd_syntax}:SERnumber")
        self._type = ChannelProbeIdType(device, f"{self._cmd_syntax}:TYPE")

    @property
    def sernumber(self) -> ChannelProbeIdSernumber:
        """Return the ``CH<x>:PRObe:ID:SERnumber`` command.

        Description:
            - Returns the serial number of the probe attached to channel <x>, where x is the channel
              number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:ID:SERnumber?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:ID:SERnumber?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:ID:SERnumber?
            ```
        """
        return self._sernumber

    @property
    def type(self) -> ChannelProbeIdType:
        """Return the ``CH<x>:PRObe:ID:TYPE`` command.

        Description:
            - Returns the type of probe attached to the channel specified by <x>, where x is the
              channel number. Level 2 (or higher) probes supply their exact product nomenclature;
              for Level 0 or 1 probes, a generic 'No Probe Detected message is returned.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:ID:TYPE?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:ID:TYPE?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:ID:TYPE?
            ```
        """
        return self._type


class ChannelProbeGain(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PRObe:GAIN`` command.

    Description:
        - This command specifies the gain factor for the probe attached to the channel specified by
          <x>, where x is the channel number. The 'gain' of a probe is the output divided by the
          input transfer ratio. For example, a common 10x probe has a gain of 0.1 V.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:GAIN?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:GAIN?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:GAIN value`` command.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:GAIN <NR3>
        - CH<x>:PRObe:GAIN?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the probe gain. Allowed values depend
          on the specific probe.
    """


class ChannelProbeForcedrange(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PRObe:FORCEDRange`` command.

    Description:
        - This command specifies the range of a TekVPI probe attached to the channel specified by
          <x>, where x is the channel number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:FORCEDRange?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:FORCEDRange?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:FORCEDRange value``
          command.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:FORCEDRange <NR3>
        - CH<x>:PRObe:FORCEDRange?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is a floating point number that specifies the range, which is probe specific.
    """


class ChannelProbeDegaussState(SCPICmdRead):
    """The ``CH<x>:PRObe:DEGAUss:STATE`` command.

    Description:
        - This command returns the state of the probe degauss for the channel specified by <x>,
          where is x is the channel number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe:DEGAUss:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:DEGAUss:STATE?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:DEGAUss:STATE?
        ```
    """


class ChannelProbeDegauss(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PRObe:DEGAUss`` command.

    Description:
        - This command starts a degauss auto-zero cycle on a TekVPI current probe attached to the
          input channel specified by <x>, where x is the channel number.

    Usage:
        - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:DEGAUss value`` command.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:DEGAUss EXECute
        ```

    Info:
        - ``EXECute`` initiates the degauss operation.

    Properties:
        - ``.state``: The ``CH<x>:PRObe:DEGAUss:STATE`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = ChannelProbeDegaussState(device, f"{self._cmd_syntax}:STATE")

    @property
    def state(self) -> ChannelProbeDegaussState:
        """Return the ``CH<x>:PRObe:DEGAUss:STATE`` command.

        Description:
            - This command returns the state of the probe degauss for the channel specified by <x>,
              where is x is the channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:DEGAUss:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:DEGAUss:STATE?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:DEGAUss:STATE?
            ```
        """
        return self._state


class ChannelProbeCommand(SCPICmdWrite):
    """The ``CH<x>:PRObe:COMMAND`` command.

    Description:
        - Sets the state of the probe control specified with the first argument to the state
          specified with the second argument. The commands and states are unique to the attached
          probe. Only certain VPI probes support this command. See the specific probe documentation
          for how to set these string arguments. The command form takes 2 string arguments: the
          first is the probe command enumeration and the second is the data value. The query form
          requires a single quoted string argument to specify the probe command enumeration for
          which the response data is requested.

    Usage:
        - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:COMMAND value`` command.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:COMMAND <QString>, <QString>
        ```

    Info:
        - ``<QString>`` are quoted strings specifying the probe command and value to set in the
          probe attached to the specified channel.
    """


class ChannelProbeAutozero(SCPICmdWrite):
    """The ``CH<x>:PRObe:AUTOZero`` command.

    Description:
        - This command executes the attached probe's Auto Zero function, for probes that support
          this feature. See your probe documentation for more details.

    Usage:
        - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:AUTOZero value`` command.

    SCPI Syntax:
        ```
        - CH<x>:PRObe:AUTOZero EXECute
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``EXECute`` sets the probe attached to the specified channel to autozero.
    """


#  pylint: disable=too-many-instance-attributes
class ChannelProbe(SCPICmdRead):
    """The ``CH<x>:PRObe`` command.

    Description:
        - Returns the gain factor of the probe attached to channel <x>, where x is the channel
          number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PRObe?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:PRObe?
        ```

    Properties:
        - ``.autozero``: The ``CH<x>:PRObe:AUTOZero`` command.
        - ``.command``: The ``CH<x>:PRObe:COMMAND`` command.
        - ``.degauss``: The ``CH<x>:PRObe:DEGAUss`` command.
        - ``.forcedrange``: The ``CH<x>:PRObe:FORCEDRange`` command.
        - ``.gain``: The ``CH<x>:PRObe:GAIN`` command.
        - ``.id``: The ``CH<x>:PRObe:ID`` command.
        - ``.resistance``: The ``CH<x>:PRObe:RESistance`` command.
        - ``.signal``: The ``CH<x>:PRObe:SIGnal`` command.
        - ``.units``: The ``CH<x>:PRObe:UNIts`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._autozero = ChannelProbeAutozero(device, f"{self._cmd_syntax}:AUTOZero")
        self._command = ChannelProbeCommand(device, f"{self._cmd_syntax}:COMMAND")
        self._degauss = ChannelProbeDegauss(device, f"{self._cmd_syntax}:DEGAUss")
        self._forcedrange = ChannelProbeForcedrange(device, f"{self._cmd_syntax}:FORCEDRange")
        self._gain = ChannelProbeGain(device, f"{self._cmd_syntax}:GAIN")
        self._id = ChannelProbeId(device, f"{self._cmd_syntax}:ID")
        self._resistance = ChannelProbeResistance(device, f"{self._cmd_syntax}:RESistance")
        self._signal = ChannelProbeSignal(device, f"{self._cmd_syntax}:SIGnal")
        self._units = ChannelProbeUnits(device, f"{self._cmd_syntax}:UNIts")

    @property
    def autozero(self) -> ChannelProbeAutozero:
        """Return the ``CH<x>:PRObe:AUTOZero`` command.

        Description:
            - This command executes the attached probe's Auto Zero function, for probes that support
              this feature. See your probe documentation for more details.

        Usage:
            - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:AUTOZero value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:AUTOZero EXECute
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``EXECute`` sets the probe attached to the specified channel to autozero.
        """
        return self._autozero

    @property
    def command(self) -> ChannelProbeCommand:
        """Return the ``CH<x>:PRObe:COMMAND`` command.

        Description:
            - Sets the state of the probe control specified with the first argument to the state
              specified with the second argument. The commands and states are unique to the attached
              probe. Only certain VPI probes support this command. See the specific probe
              documentation for how to set these string arguments. The command form takes 2 string
              arguments: the first is the probe command enumeration and the second is the data
              value. The query form requires a single quoted string argument to specify the probe
              command enumeration for which the response data is requested.

        Usage:
            - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:COMMAND value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:COMMAND <QString>, <QString>
            ```

        Info:
            - ``<QString>`` are quoted strings specifying the probe command and value to set in the
              probe attached to the specified channel.
        """
        return self._command

    @property
    def degauss(self) -> ChannelProbeDegauss:
        """Return the ``CH<x>:PRObe:DEGAUss`` command.

        Description:
            - This command starts a degauss auto-zero cycle on a TekVPI current probe attached to
              the input channel specified by <x>, where x is the channel number.

        Usage:
            - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:DEGAUss value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:DEGAUss EXECute
            ```

        Info:
            - ``EXECute`` initiates the degauss operation.

        Sub-properties:
            - ``.state``: The ``CH<x>:PRObe:DEGAUss:STATE`` command.
        """
        return self._degauss

    @property
    def forcedrange(self) -> ChannelProbeForcedrange:
        """Return the ``CH<x>:PRObe:FORCEDRange`` command.

        Description:
            - This command specifies the range of a TekVPI probe attached to the channel specified
              by <x>, where x is the channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:FORCEDRange?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:FORCEDRange?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:FORCEDRange value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:FORCEDRange <NR3>
            - CH<x>:PRObe:FORCEDRange?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is a floating point number that specifies the range, which is probe
              specific.
        """
        return self._forcedrange

    @property
    def gain(self) -> ChannelProbeGain:
        """Return the ``CH<x>:PRObe:GAIN`` command.

        Description:
            - This command specifies the gain factor for the probe attached to the channel specified
              by <x>, where x is the channel number. The 'gain' of a probe is the output divided by
              the input transfer ratio. For example, a common 10x probe has a gain of 0.1 V.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:GAIN?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:GAIN?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:GAIN value`` command.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:GAIN <NR3>
            - CH<x>:PRObe:GAIN?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the probe gain. Allowed values
              depend on the specific probe.
        """
        return self._gain

    @property
    def id(self) -> ChannelProbeId:
        """Return the ``CH<x>:PRObe:ID`` command.

        Description:
            - Returns the type and serial number of the probe attached to channel <x>, where x is
              the channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:ID?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:ID?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:ID?
            ```

        Sub-properties:
            - ``.sernumber``: The ``CH<x>:PRObe:ID:SERnumber`` command.
            - ``.type``: The ``CH<x>:PRObe:ID:TYPE`` command.
        """
        return self._id

    @property
    def resistance(self) -> ChannelProbeResistance:
        """Return the ``CH<x>:PRObe:RESistance`` command.

        Description:
            - Returns the resistance factor of the probe attached to channel <x>, where x is the
              channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:RESistance?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:RESistance?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:RESistance?
            ```
        """
        return self._resistance

    @property
    def signal(self) -> ChannelProbeSignal:
        """Return the ``CH<x>:PRObe:SIGnal`` command.

        Description:
            - This command specifies the input bypass setting of a TekVPI probe attached to channel
              <x>, where x is the channel number. The probe must support input bypass, for example
              TCP0001. This command is ignored if sent to an unsupported probe.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:SIGnal?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:SIGnal?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:PRObe:SIGnal value`` command.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:SIGnal {BYPass|PASS}
            - CH<x>:PRObe:SIGnal?
            ```

        Info:
            - ``BYPass`` sets the probe to Bypass mode.
            - ``PASS`` sets the probe to Pass mode.
        """
        return self._signal

    @property
    def units(self) -> ChannelProbeUnits:
        """Return the ``CH<x>:PRObe:UNIts`` command.

        Description:
            - Returns a string describing the units of measure for the probe attached to channel
              <x>, where x is the channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe:UNIts?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe:UNIts?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:PRObe:UNIts?
            ```
        """
        return self._units


class ChannelPosition(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:POSition`` command.

    Description:
        - This command specifies the vertical position of channel <x>, where x is the channel number
          (1-4). The position value is applied to the signal before it is digitized. Increasing the
          position value of a waveform causes the waveform to move up. Decreasing the position value
          causes the waveform to move down. The position value determines the vertical graticule
          coordinate at which input signal values, minus the present offset setting for that
          channel, are displayed. For example, if the position for Channel 3 is set to 2.0 and the
          offset is set to 3.0, then input signals equal to 3.0 units are displayed 2.0 divisions
          above the center of the screen (at 1 V/div).

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:POSition?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:POSition value`` command.

    SCPI Syntax:
        ```
        - CH<x>:POSition <NR3>
        - CH<x>:POSition?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the position value for channel <x>, in
          divisions, from the center graticule. The range is 8 to -8 divisions.
    """


class ChannelOffset(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:OFFSet`` command.

    Description:
        - This command sets or queries the vertical offset for the specified analog channel.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:OFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:OFFSet?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:OFFSet value`` command.

    SCPI Syntax:
        ```
        - CH<x>:OFFSet <NR3>
        - CH<x>:OFFSet?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the offset value for the specified channel.
    """


class ChannelLabel(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel`` command.

    Description:
        - This command specifies the waveform label for channel <x>, where x is the channel number
          (1-4).

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel <Qstring>
        - CH<x>:LABel?
        ```

    Info:
        - ``<Qstring>`` is an alphanumeric string of text, enclosed in quotes, that contains the
          text label information for the channel <x> waveform. The text string is limited to 30
          characters.
    """


class ChannelInvert(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:INVert`` command.

    Description:
        - This command specifies the invert function for channel <x>, where is the channel number
          (1-4) . When on, the invert function inverts the waveform for the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:INVert?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:INVert?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:INVert value`` command.

    SCPI Syntax:
        ```
        - CH<x>:INVert {ON|OFF}
        - CH<x>:INVert?
        ```

    Info:
        - ``OFF`` sets the invert function for channel <x> to off.
        - ``ON`` sets the invert function for channel <x> to on.
    """


class ChannelImpedance(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``CH<x>:IMPedance`` command.

    Description:
        - Sets or returns the input impedance of channel <x>, where x is the channel number. This
          command is provided for compatibilty.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:IMPedance?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:IMPedance?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write()`` method will send the ``CH<x>:IMPedance`` command.

    SCPI Syntax:
        ```
        - CH<x>:IMPedance
        - CH<x>:IMPedance?
        ```
    """


class ChannelDeskew(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:DESKew`` command.

    Description:
        - This command sets or queries the horizontal deskew time for the specified channel. The
          channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:DESKew?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:DESKew?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:DESKew value`` command.

    SCPI Syntax:
        ```
        - CH<x>:DESKew <NR3>
        - CH<x>:DESKew?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the deskew time for this channel, ranging from -125 ns to +125 ns with a
          resolution of 40 ps. Out-of-range values are clipped.
    """


class ChannelCoupling(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:COUPling`` command.

    Description:
        - Sets or returns the input attenuator coupling setting for channel <x>, where x is the
          channel number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:COUPling?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:COUPling?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:COUPling value`` command.

    SCPI Syntax:
        ```
        - CH<x>:COUPling {AC|DC|GND}
        - CH<x>:COUPling?
        ```

    Info:
        - ``AC`` sets channel <x> to AC coupling.
        - ``DC`` sets channel <x> to DC coupling.
        - ``GND`` sets channel<x> to ground. Only a flat, ground-level waveform will be displayed.
    """


class ChannelBandwidth(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:BANdwidth`` command.

    Description:
        - Sets or returns the selectable low-pass bandwidth limit filter for channel <x>, where x is
          the channel number.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:BANdwidth?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:BANdwidth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:BANdwidth value`` command.

    SCPI Syntax:
        ```
        - CH<x>:BANdwidth {TWEnty|FULl|<NR3>}
        - CH<x>:BANdwidth?
        ```

    Info:
        - ``TWEnty`` sets the upper bandwidth limit of channel <x> to 20 MHz.
        - ``FULl`` disables any optional bandwidth limiting. The specified channel operates at its
          maximum attainable bandwidth.
        - ``<NR3>`` is a double-precision ASCII string. The oscilloscope rounds this value to an
          available bandwidth using geometric rounding, and then uses this value to set the upper
          bandwidth limit.
    """


#  pylint: disable=too-many-instance-attributes
class Channel(ValidatedChannel, SCPICmdRead):
    """The ``CH<x>`` command.

    Description:
        - This query-only command returns the vertical parameters for the specified channel. The
          channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>?
        ```

    Properties:
        - ``.bandwidth``: The ``CH<x>:BANdwidth`` command.
        - ``.coupling``: The ``CH<x>:COUPling`` command.
        - ``.deskew``: The ``CH<x>:DESKew`` command.
        - ``.impedance``: The ``CH<x>:IMPedance`` command.
        - ``.invert``: The ``CH<x>:INVert`` command.
        - ``.label``: The ``CH<x>:LABel`` command.
        - ``.offset``: The ``CH<x>:OFFSet`` command.
        - ``.position``: The ``CH<x>:POSition`` command.
        - ``.probe``: The ``CH<x>:PRObe`` command.
        - ``.scale``: The ``CH<x>:SCAle`` command.
        - ``.termination``: The ``CH<x>:TERmination`` command.
        - ``.volts``: The ``CH<x>:VOLts`` command.
        - ``.yunits``: The ``CH<x>:YUNits`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "CH<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._bandwidth = ChannelBandwidth(device, f"{self._cmd_syntax}:BANdwidth")
        self._coupling = ChannelCoupling(device, f"{self._cmd_syntax}:COUPling")
        self._deskew = ChannelDeskew(device, f"{self._cmd_syntax}:DESKew")
        self._impedance = ChannelImpedance(device, f"{self._cmd_syntax}:IMPedance")
        self._invert = ChannelInvert(device, f"{self._cmd_syntax}:INVert")
        self._label = ChannelLabel(device, f"{self._cmd_syntax}:LABel")
        self._offset = ChannelOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._position = ChannelPosition(device, f"{self._cmd_syntax}:POSition")
        self._probe = ChannelProbe(device, f"{self._cmd_syntax}:PRObe")
        self._scale = ChannelScale(device, f"{self._cmd_syntax}:SCAle")
        self._termination = ChannelTermination(device, f"{self._cmd_syntax}:TERmination")
        self._volts = ChannelVolts(device, f"{self._cmd_syntax}:VOLts")
        self._yunits = ChannelYunits(device, f"{self._cmd_syntax}:YUNits")

    @property
    def bandwidth(self) -> ChannelBandwidth:
        """Return the ``CH<x>:BANdwidth`` command.

        Description:
            - Sets or returns the selectable low-pass bandwidth limit filter for channel <x>, where
              x is the channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:BANdwidth?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:BANdwidth?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:BANdwidth value`` command.

        SCPI Syntax:
            ```
            - CH<x>:BANdwidth {TWEnty|FULl|<NR3>}
            - CH<x>:BANdwidth?
            ```

        Info:
            - ``TWEnty`` sets the upper bandwidth limit of channel <x> to 20 MHz.
            - ``FULl`` disables any optional bandwidth limiting. The specified channel operates at
              its maximum attainable bandwidth.
            - ``<NR3>`` is a double-precision ASCII string. The oscilloscope rounds this value to an
              available bandwidth using geometric rounding, and then uses this value to set the
              upper bandwidth limit.
        """
        return self._bandwidth

    @property
    def coupling(self) -> ChannelCoupling:
        """Return the ``CH<x>:COUPling`` command.

        Description:
            - Sets or returns the input attenuator coupling setting for channel <x>, where x is the
              channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:COUPling?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:COUPling?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:COUPling value`` command.

        SCPI Syntax:
            ```
            - CH<x>:COUPling {AC|DC|GND}
            - CH<x>:COUPling?
            ```

        Info:
            - ``AC`` sets channel <x> to AC coupling.
            - ``DC`` sets channel <x> to DC coupling.
            - ``GND`` sets channel<x> to ground. Only a flat, ground-level waveform will be
              displayed.
        """
        return self._coupling

    @property
    def deskew(self) -> ChannelDeskew:
        """Return the ``CH<x>:DESKew`` command.

        Description:
            - This command sets or queries the horizontal deskew time for the specified channel. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:DESKew?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:DESKew?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:DESKew value`` command.

        SCPI Syntax:
            ```
            - CH<x>:DESKew <NR3>
            - CH<x>:DESKew?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the deskew time for this channel, ranging from -125 ns to +125 ns with a
              resolution of 40 ps. Out-of-range values are clipped.
        """
        return self._deskew

    @property
    def impedance(self) -> ChannelImpedance:
        """Return the ``CH<x>:IMPedance`` command.

        Description:
            - Sets or returns the input impedance of channel <x>, where x is the channel number.
              This command is provided for compatibilty.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:IMPedance?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:IMPedance?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write()`` method will send the ``CH<x>:IMPedance`` command.

        SCPI Syntax:
            ```
            - CH<x>:IMPedance
            - CH<x>:IMPedance?
            ```
        """
        return self._impedance

    @property
    def invert(self) -> ChannelInvert:
        """Return the ``CH<x>:INVert`` command.

        Description:
            - This command specifies the invert function for channel <x>, where is the channel
              number (1-4) . When on, the invert function inverts the waveform for the specified
              channel.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:INVert?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:INVert?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:INVert value`` command.

        SCPI Syntax:
            ```
            - CH<x>:INVert {ON|OFF}
            - CH<x>:INVert?
            ```

        Info:
            - ``OFF`` sets the invert function for channel <x> to off.
            - ``ON`` sets the invert function for channel <x> to on.
        """
        return self._invert

    @property
    def label(self) -> ChannelLabel:
        """Return the ``CH<x>:LABel`` command.

        Description:
            - This command specifies the waveform label for channel <x>, where x is the channel
              number (1-4).

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel value`` command.

        SCPI Syntax:
            ```
            - CH<x>:LABel <Qstring>
            - CH<x>:LABel?
            ```

        Info:
            - ``<Qstring>`` is an alphanumeric string of text, enclosed in quotes, that contains the
              text label information for the channel <x> waveform. The text string is limited to 30
              characters.
        """
        return self._label

    @property
    def offset(self) -> ChannelOffset:
        """Return the ``CH<x>:OFFSet`` command.

        Description:
            - This command sets or queries the vertical offset for the specified analog channel.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:OFFSet?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:OFFSet value`` command.

        SCPI Syntax:
            ```
            - CH<x>:OFFSet <NR3>
            - CH<x>:OFFSet?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the offset value for the specified channel.
        """
        return self._offset

    @property
    def position(self) -> ChannelPosition:
        """Return the ``CH<x>:POSition`` command.

        Description:
            - This command specifies the vertical position of channel <x>, where x is the channel
              number (1-4). The position value is applied to the signal before it is digitized.
              Increasing the position value of a waveform causes the waveform to move up. Decreasing
              the position value causes the waveform to move down. The position value determines the
              vertical graticule coordinate at which input signal values, minus the present offset
              setting for that channel, are displayed. For example, if the position for Channel 3 is
              set to 2.0 and the offset is set to 3.0, then input signals equal to 3.0 units are
              displayed 2.0 divisions above the center of the screen (at 1 V/div).

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:POSition?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:POSition value`` command.

        SCPI Syntax:
            ```
            - CH<x>:POSition <NR3>
            - CH<x>:POSition?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the position value for channel
              <x>, in divisions, from the center graticule. The range is 8 to -8 divisions.
        """
        return self._position

    @property
    def probe(self) -> ChannelProbe:
        """Return the ``CH<x>:PRObe`` command.

        Description:
            - Returns the gain factor of the probe attached to channel <x>, where x is the channel
              number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PRObe?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PRObe?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:PRObe?
            ```

        Sub-properties:
            - ``.autozero``: The ``CH<x>:PRObe:AUTOZero`` command.
            - ``.command``: The ``CH<x>:PRObe:COMMAND`` command.
            - ``.degauss``: The ``CH<x>:PRObe:DEGAUss`` command.
            - ``.forcedrange``: The ``CH<x>:PRObe:FORCEDRange`` command.
            - ``.gain``: The ``CH<x>:PRObe:GAIN`` command.
            - ``.id``: The ``CH<x>:PRObe:ID`` command.
            - ``.resistance``: The ``CH<x>:PRObe:RESistance`` command.
            - ``.signal``: The ``CH<x>:PRObe:SIGnal`` command.
            - ``.units``: The ``CH<x>:PRObe:UNIts`` command.
        """
        return self._probe

    @property
    def scale(self) -> ChannelScale:
        """Return the ``CH<x>:SCAle`` command.

        Description:
            - This command sets or returns the vertical scale for the specified analog channel. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:SCAle?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:SCAle value`` command.

        SCPI Syntax:
            ```
            - CH<x>:SCAle <NR3>
            - CH<x>:SCAle?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the vertical scale for the specified analog channel.
        """
        return self._scale

    @property
    def termination(self) -> ChannelTermination:
        """Return the ``CH<x>:TERmination`` command.

        Description:
            - Sets the connected-disconnected status of a 50 Ω resistor, which may be connected
              between the specified channel's coupled input and oscilloscope ground. The channel is
              specified by <x>. There is also a corresponding query that requests the termination
              parameter and translates this enumeration into one of the two float values. This
              command is maintained for compatibility.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:TERmination?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:TERmination?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write()`` method will send the ``CH<x>:TERmination`` command.

        SCPI Syntax:
            ```
            - CH<x>:TERmination
            - CH<x>:TERmination?
            ```
        """
        return self._termination

    @property
    def volts(self) -> ChannelVolts:
        """Return the ``CH<x>:VOLts`` command.

        Description:
            - This command specifies the vertical sensitivity for channel <x>, where x is the
              channel number.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:VOLts?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:VOLts?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:VOLts value`` command.

        SCPI Syntax:
            ```
            - CH<x>:VOLts <NR3>
            - CH<x>:VOLts?
            ```

        Info:
            - ``<NR3>`` is the vertical sensitivity, in volts.
        """
        return self._volts

    @property
    def yunits(self) -> ChannelYunits:
        """Return the ``CH<x>:YUNits`` command.

        Description:
            - This command specifies the vertical units for the channel specified by <x>, where x is
              the channel number. String arguments are case insensitive and any unsupported units
              will generate an error. Supported units are: %, /Hz, A, A/A, A/V, A/W, A/dB, A/s, AA,
              AW, AdB, As, B, Hz, IRE, S/s, V, V/A, V/V, V/W, V/dB, V/s, VV, VW, VdB, volts, Vs, W,
              W/A, W/V, W/W, W/dB, W/s, WA, WV, WW, WdB, Ws, dB, dB/A, dB/V, dB/W, dB/dB, dBA, dBV,
              dBW, dBdB, day, degrees, div, hr, min, ohms, percent, s The vertical units affect the
              'Probe Type' that is shown in the 'Probe Setup' menu: Setting ``CH<x>:YUNits`` to 'V'
              causes the probe type to be displayed as 'Voltage'. When ``CH1:AMSVIAVOLTs:ENAble`` is
              set to OFF, setting ``CH<x>:YUNits`` to 'A' causes the probe type to be displayed as
              'Current'. Setting ``CH<x>:YUNits`` to anything else causes the probe type not to be
              displayed (neither 'Voltage' nor 'Current' are highlighted).

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:YUNits?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:YUNits?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:YUNits value`` command.

        SCPI Syntax:
            ```
            - CH<x>:YUNits <QString>
            - CH<x>:YUNits?
            ```

        Info:
            - ``QString`` is a string of text surrounded by quotes, specifying the supported units.
        """
        return self._yunits



"""The MPSU50_2ST commands module.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""

from typing import Any, Dict

from .gen_3vhopd_mpsu.firmware import Firmware
from .gen_3vhopd_mpsu.psu import PsuItem
from .gen_3vhopd_mpsu.status import Status
from .gen_3vhopd_mpsu.trigger import Trigger
from .gen_4ajvxe_mpmpsu.buffervar import Buffervar


class MPSU50_2STMeta(type):  # noqa: N801
    """A metaclass that prevents instantiation of the class it's applied to."""

    def __call__(cls, *_: Any, **__: Any) -> None:
        """Raises a NotImplementedError when an attempt is made to instantiate the class."""
        instantiation_error_msg = (
            "This provides access to all the commands for the MPSU50_2ST device."
            "It serves only for type-hinting purposes and should not be instantiated."
        )
        raise NotImplementedError(instantiation_error_msg)


class MPSU50_2STCommands(metaclass=MPSU50_2STMeta):  # noqa: N801
    """The MPSU50-2ST commands.

    This provides access to all the commands for the MPSU50-2ST device. See the documentation of
    each property for more usage information.

    Properties and methods:
        - ``.buffer_var``: The ``bufferVar`` command tree.
        - ``.firmware``: The ``firmware`` command tree.
        - ``.license``: The ``license`` attribute.
        - ``.manufacturer``: The ``manufacturer`` attribute.
        - ``.model``: The ``model`` attribute.
        - ``.psu``: The ``psu[X]`` command tree.
        - ``.serialno``: The ``serialno`` attribute.
        - ``.status``: The ``status`` command tree.
        - ``.trigger``: The ``trigger`` command tree.
        - ``.version``: The ``version`` attribute.
    """

    @property
    def buffer_var(self) -> Dict[str, Buffervar]:  # pyright: ignore[reportReturnType]
        """Return the ``bufferVar`` command tree.

        Info:
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Sub-properties and sub-methods:
            - ``.appendmode``: The ``bufferVar.appendmode`` attribute.
            - ``.cachemode``: The ``bufferVar.cachemode`` attribute.
            - ``.capacity``: The ``bufferVar.capacity`` attribute.
            - ``.clear()``: The ``bufferVar.clear()`` function.
            - ``.clearcache()``: The ``bufferVar.clearcache()`` function.
            - ``.columnname``: The ``bufferVar.columnname`` command tree.
            - ``.fillcount``: The ``bufferVar.fillcount`` attribute.
            - ``.fillmode``: The ``bufferVar.fillmode`` attribute.
            - ``.fractionalseconds``: The ``bufferVar.fractionalseconds[N]`` attribute.
            - ``.measurefunctions``: The ``bufferVar.measurefunctions[N]`` attribute.
            - ``.measureranges``: The ``bufferVar.measureranges[N]`` attribute.
            - ``.n``: The ``bufferVar.n`` attribute.
            - ``.readings``: The ``bufferVar.readings[N]`` attribute.
            - ``.seconds``: The ``bufferVar.seconds[N]`` attribute.
            - ``.sourcefunctions``: The ``bufferVar.sourcefunctions[N]`` attribute.
            - ``.sourceoutputstates``: The ``bufferVar.sourceoutputstates[N]`` attribute.
            - ``.sourceranges``: The ``bufferVar.sourceranges[N]`` attribute.
            - ``.sourcevalues``: The ``bufferVar.sourcevalues[N]`` attribute.
            - ``.statuses``: The ``bufferVar.statuses[N]`` attribute.
            - ``.timestampresolution``: The ``bufferVar.timestampresolution`` attribute.
            - ``.timestamps``: The ``bufferVar.timestamps[N]`` attribute.
        """

    @property
    def firmware(self) -> Firmware:  # pyright: ignore[reportReturnType]
        """Return the ``firmware`` command tree.

        Sub-properties and sub-methods:
            - ``.update()``: The ``firmware.update()`` function.
            - ``.verify()``: The ``firmware.verify()`` function.
        """

    @property
    def license(self) -> str:  # pyright: ignore[reportReturnType]
        """Return the ``license`` attribute.

        Description:
            - This attribute stores the ``license`` of the module installed in the specified
              mainframe slot.

        Usage:
            - Accessing this property will send the ``print(license)`` query.

        TSP Syntax:
            ```
            - print(license)
            ```

        Info:
            - ``license``, the license text of the instrument.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """

    @property
    def manufacturer(self) -> str:  # pyright: ignore[reportReturnType]
        """Return the ``manufacturer`` attribute.

        Description:
            - This attribute stores the ``manufacturer`` of the module in the specified mainframe
              slot.

        Usage:
            - Accessing this property will send the ``print(manufacturer)`` query.

        TSP Syntax:
            ```
            - print(manufacturer)
            ```

        Info:
            - ``manufacturer``, this is always Tektronix.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """

    @property
    def model(self) -> str:  # pyright: ignore[reportReturnType]
        """Return the ``model`` attribute.

        Description:
            - This attribute stores the ``model`` number of the module in the specified mainframe
              slot.

        Usage:
            - Accessing this property will send the ``print(model)`` query.

        TSP Syntax:
            ```
            - print(model)
            ```

        Info:
            - ``model``, the ``model`` number of the module.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """

    @property
    def psu(self) -> Dict[int, PsuItem]:  # pyright: ignore[reportReturnType]
        """Return the ``psu[X]`` command tree.

        Info:
            - ``X``, the module channel number.

        Sub-properties and sub-methods:
            - ``.abort()``: The ``psu[X].abort()`` function.
            - ``.configlist``: The ``psu[X].configlist`` command tree.
            - ``.defbuffer1``: The ``psu[X].defbuffer1`` attribute.
            - ``.defbuffer2``: The ``psu[X].defbuffer2`` attribute.
            - ``.makebuffer()``: The ``psu[X].makebuffer()`` function.
            - ``.measure``: The ``psu[X].measure`` command tree.
            - ``.overlapped``: The ``psu[X].overlapped`` attribute.
            - ``.reset()``: The ``psu[X].reset()`` function.
            - ``.source``: The ``psu[X].source`` command tree.
            - ``.trigger``: The ``psu[X].trigger`` command tree.
            - ``.waitcomplete()``: The ``psu[X].waitcomplete()`` function.
        """

    @property
    def serialno(self) -> str:  # pyright: ignore[reportReturnType]
        """Return the ``serialno`` attribute.

        Description:
            - This attribute stores the serial number of the module in the specified mainframe slot.

        Usage:
            - Accessing this property will send the ``print(serialno)`` query.

        TSP Syntax:
            ```
            - print(serialno)
            ```

        Info:
            - ``serialno``, the serial number of the module.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """

    @property
    def status(self) -> Status:  # pyright: ignore[reportReturnType]
        """Return the ``status`` command tree.

        Sub-properties and sub-methods:
            - ``.measurement``: The ``status.measurement`` command tree.
            - ``.operation``: The ``status.operation`` command tree.
            - ``.questionable``: The ``status.questionable`` command tree.
            - ``.reset()``: The ``status.reset()`` function.
        """

    @property
    def trigger(self) -> Trigger:  # pyright: ignore[reportReturnType]
        """Return the ``trigger`` command tree.

        Sub-properties and sub-methods:
            - ``.model``: The ``trigger.model`` command tree.
        """

    @property
    def version(self) -> str:  # pyright: ignore[reportReturnType]
        """Return the ``version`` attribute.

        Description:
            - This attribute stores the firmware ``version`` number of the module in the specified
              slot of the mainframe.

        Usage:
            - Accessing this property will send the ``print(version)`` query.

        TSP Syntax:
            ```
            - print(version)
            ```

        Info:
            - ``version``, the module firmware ``version`` number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """



"""The math1 commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MATH1:AUTOSCale {ON|OFF|1|0}
    - MATH1:AUTOSCale?
    - MATH1:DEFine <QString>
    - MATH1:DEFine?
    - MATH1:HORizontal:POSition <NR3>
    - MATH1:HORizontal:POSition?
    - MATH1:HORizontal:SCAle <NR3>
    - MATH1:HORizontal:SCAle?
    - MATH1:HORizontal:UNIts?
    - MATH1:LABel <QString>
    - MATH1:LABel?
    - MATH1:SPECTral:MAG {LINEAr|DB}
    - MATH1:SPECTral:MAG?
    - MATH1:SPECTral:WINdow {RECTangular|HAMming|HANning|BLAckmanharris}
    - MATH1:SPECTral:WINdow?
    - MATH1:TYPe {DUAL|FFT|ADVanced|SPECTRUM}
    - MATH1:TYPe?
    - MATH1:VERTical:POSition <NR3>
    - MATH1:VERTical:POSition?
    - MATH1:VERTical:SCAle <NR3>
    - MATH1:VERTical:SCAle?
    - MATH1:VERTical:UNIts?
    - MATH1?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Math1VerticalUnits(SCPICmdRead):
    """The ``MATH1:VERTical:UNIts`` command.

    Description:
        - Returns the math waveform vertical measurement unit value.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:VERTical:UNIts?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:VERTical:UNIts?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MATH1:VERTical:UNIts?
        ```
    """


class Math1VerticalScale(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:VERTical:SCAle`` command.

    Description:
        - This command specifies the vertical scale of the currently selected math type. This
          setting controls the display only, graphically scaling these waveforms and having no
          affect on the acquisition hardware. For a signal with constant amplitude, increasing the
          scale causes the waveform to be displayed smaller. Decreasing the scale causes the
          waveform to be displayed larger.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:VERTical:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:VERTical:SCAle?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:VERTical:SCAle value`` command.

    SCPI Syntax:
        ```
        - MATH1:VERTical:SCAle <NR3>
        - MATH1:VERTical:SCAle?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the scale-per-division in the current
          math vertical units. The range is from 1.0E-12 through 500.0E+12.
    """


class Math1VerticalPosition(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:VERTical:POSition`` command.

    Description:
        - This command specifies the vertical position of the currently selected math type.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:VERTical:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:VERTical:POSition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:VERTical:POSition value``
          command.

    SCPI Syntax:
        ```
        - MATH1:VERTical:POSition <NR3>
        - MATH1:VERTical:POSition?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the desired position in divisions from
          the center graticule.
    """


class Math1Vertical(SCPICmdRead):
    """The ``MATH1:VERTical`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:VERTical?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:VERTical?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.position``: The ``MATH1:VERTical:POSition`` command.
        - ``.scale``: The ``MATH1:VERTical:SCAle`` command.
        - ``.units``: The ``MATH1:VERTical:UNIts`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._position = Math1VerticalPosition(device, f"{self._cmd_syntax}:POSition")
        self._scale = Math1VerticalScale(device, f"{self._cmd_syntax}:SCAle")
        self._units = Math1VerticalUnits(device, f"{self._cmd_syntax}:UNIts")

    @property
    def position(self) -> Math1VerticalPosition:
        """Return the ``MATH1:VERTical:POSition`` command.

        Description:
            - This command specifies the vertical position of the currently selected math type.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:VERTical:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:VERTical:POSition?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:VERTical:POSition value``
              command.

        SCPI Syntax:
            ```
            - MATH1:VERTical:POSition <NR3>
            - MATH1:VERTical:POSition?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the desired position in divisions
              from the center graticule.
        """
        return self._position

    @property
    def scale(self) -> Math1VerticalScale:
        """Return the ``MATH1:VERTical:SCAle`` command.

        Description:
            - This command specifies the vertical scale of the currently selected math type. This
              setting controls the display only, graphically scaling these waveforms and having no
              affect on the acquisition hardware. For a signal with constant amplitude, increasing
              the scale causes the waveform to be displayed smaller. Decreasing the scale causes the
              waveform to be displayed larger.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:VERTical:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:VERTical:SCAle?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:VERTical:SCAle value``
              command.

        SCPI Syntax:
            ```
            - MATH1:VERTical:SCAle <NR3>
            - MATH1:VERTical:SCAle?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the scale-per-division in the
              current math vertical units. The range is from 1.0E-12 through 500.0E+12.
        """
        return self._scale

    @property
    def units(self) -> Math1VerticalUnits:
        """Return the ``MATH1:VERTical:UNIts`` command.

        Description:
            - Returns the math waveform vertical measurement unit value.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:VERTical:UNIts?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:VERTical:UNIts?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MATH1:VERTical:UNIts?
            ```
        """
        return self._units


class Math1Type(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:TYPe`` command.

    Description:
        - This command specifies the math type ( DUAL, FFT, ADVanced or SPECTRUM ). This command is
          used along with ``MATH1:DEFINE``, which specifies the current mathematical expression as a
          text string which defines the waveform. You must specify the math type before defining the
          math expression. For a list of math expressions, see ``MATH1:DEFINE``.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:TYPe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:TYPe value`` command.

    SCPI Syntax:
        ```
        - MATH1:TYPe {DUAL|FFT|ADVanced|SPECTRUM}
        - MATH1:TYPe?
        ```

    Info:
        - ``DUAL`` sets the type to dual waveform math, which can use any combination of live analog
          or reference waveforms in the time domain display.
        - ``FFT`` sets the type to FFT math, which can use any live analog or reference waveform in
          the time domain.
        - ``ADVanced`` sets the type to advanced math.
        - ``SPECTRUM`` sets the type to spectrum, which can use any combination of live RF or
          reference traces in the frequency domain. This command requires option 3-SA3.
    """


class Math1SpectralWindow(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:SPECTral:WINdow`` command.

    Description:
        - This command specifies the window function for the spectral analyzer input data for the
          specified math waveform. A spectral window determines what the filter shape of the
          spectral analyzer will be in the frequency domain. It can be described by a mathematical
          function that is multiplied point-by-point times the input data to the spectral analyzer.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:SPECTral:WINdow?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:SPECTral:WINdow?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:SPECTral:WINdow value`` command.

    SCPI Syntax:
        ```
        - MATH1:SPECTral:WINdow {RECTangular|HAMming|HANning|BLAckmanharris}
        - MATH1:SPECTral:WINdow?
        ```

    Info:
        - ``RECTangular`` window function is equivalent to multiplying all gate data by one.
        - ``HAMming`` window function is based on a cosine series.
        - ``HANning`` window function is based on a cosine series.
        - ``BLAckmanharris`` window function is based on a cosine series.
    """


class Math1SpectralMag(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:SPECTral:MAG`` command.

    Description:
        - This command specifies the units of the Spectral Magnification function in the math
          string.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:SPECTral:MAG?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:SPECTral:MAG?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:SPECTral:MAG value`` command.

    SCPI Syntax:
        ```
        - MATH1:SPECTral:MAG {LINEAr|DB}
        - MATH1:SPECTral:MAG?
        ```

    Info:
        - ``LINEAR`` sets the SpectralMag units to linear.
        - ``DB`` sets the SpectralMag units to decibels.
    """


class Math1Spectral(SCPICmdRead):
    """The ``MATH1:SPECTral`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:SPECTral?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:SPECTral?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.mag``: The ``MATH1:SPECTral:MAG`` command.
        - ``.window``: The ``MATH1:SPECTral:WINdow`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._mag = Math1SpectralMag(device, f"{self._cmd_syntax}:MAG")
        self._window = Math1SpectralWindow(device, f"{self._cmd_syntax}:WINdow")

    @property
    def mag(self) -> Math1SpectralMag:
        """Return the ``MATH1:SPECTral:MAG`` command.

        Description:
            - This command specifies the units of the Spectral Magnification function in the math
              string.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:SPECTral:MAG?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:SPECTral:MAG?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:SPECTral:MAG value`` command.

        SCPI Syntax:
            ```
            - MATH1:SPECTral:MAG {LINEAr|DB}
            - MATH1:SPECTral:MAG?
            ```

        Info:
            - ``LINEAR`` sets the SpectralMag units to linear.
            - ``DB`` sets the SpectralMag units to decibels.
        """
        return self._mag

    @property
    def window(self) -> Math1SpectralWindow:
        """Return the ``MATH1:SPECTral:WINdow`` command.

        Description:
            - This command specifies the window function for the spectral analyzer input data for
              the specified math waveform. A spectral window determines what the filter shape of the
              spectral analyzer will be in the frequency domain. It can be described by a
              mathematical function that is multiplied point-by-point times the input data to the
              spectral analyzer.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:SPECTral:WINdow?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:SPECTral:WINdow?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:SPECTral:WINdow value``
              command.

        SCPI Syntax:
            ```
            - MATH1:SPECTral:WINdow {RECTangular|HAMming|HANning|BLAckmanharris}
            - MATH1:SPECTral:WINdow?
            ```

        Info:
            - ``RECTangular`` window function is equivalent to multiplying all gate data by one.
            - ``HAMming`` window function is based on a cosine series.
            - ``HANning`` window function is based on a cosine series.
            - ``BLAckmanharris`` window function is based on a cosine series.
        """
        return self._window


class Math1Label(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:LABel`` command.

    Description:
        - Sets or queries the waveform label for the math waveform.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:LABel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:LABel value`` command.

    SCPI Syntax:
        ```
        - MATH1:LABel <QString>
        - MATH1:LABel?
        ```

    Info:
        - ``<QString>`` is the quoted string used as the label for the math waveform.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Math1HorizontalUnits(SCPICmdRead):
    """The ``MATH1:HORizontal:UNIts`` command.

    Description:
        - Returns the math waveform horizontal measurement unit value.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:HORizontal:UNIts?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal:UNIts?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MATH1:HORizontal:UNIts?
        ```
    """


class Math1HorizontalScale(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:HORizontal:SCAle`` command.

    Description:
        - This command specifies the math horizontal display scale for FFT or for dual math
          waveforms that have source waveforms that are reference waveforms. The horizontal scale of
          a dual math waveform with a channel source waveform is set through the command.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:HORizontal:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal:SCAle?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:HORizontal:SCAle value`` command.

    SCPI Syntax:
        ```
        - MATH1:HORizontal:SCAle <NR3>
        - MATH1:HORizontal:SCAle?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the math horizontal scale, in seconds.
    """


class Math1HorizontalPosition(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:HORizontal:POSition`` command.

    Description:
        - This command specifies the math horizontal display position for FFT or (non-live) math
          reference waveforms.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:HORizontal:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal:POSition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:HORizontal:POSition value``
          command.

    SCPI Syntax:
        ```
        - MATH1:HORizontal:POSition <NR3>
        - MATH1:HORizontal:POSition?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the % of the math waveform that
          precedes center screen. It can vary from 0.0 to 100.0.
    """


class Math1Horizontal(SCPICmdRead):
    """The ``MATH1:HORizontal`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:HORizontal?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.position``: The ``MATH1:HORizontal:POSition`` command.
        - ``.scale``: The ``MATH1:HORizontal:SCAle`` command.
        - ``.units``: The ``MATH1:HORizontal:UNIts`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._position = Math1HorizontalPosition(device, f"{self._cmd_syntax}:POSition")
        self._scale = Math1HorizontalScale(device, f"{self._cmd_syntax}:SCAle")
        self._units = Math1HorizontalUnits(device, f"{self._cmd_syntax}:UNIts")

    @property
    def position(self) -> Math1HorizontalPosition:
        """Return the ``MATH1:HORizontal:POSition`` command.

        Description:
            - This command specifies the math horizontal display position for FFT or (non-live) math
              reference waveforms.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:HORizontal:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal:POSition?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:HORizontal:POSition value``
              command.

        SCPI Syntax:
            ```
            - MATH1:HORizontal:POSition <NR3>
            - MATH1:HORizontal:POSition?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the % of the math waveform that
              precedes center screen. It can vary from 0.0 to 100.0.
        """
        return self._position

    @property
    def scale(self) -> Math1HorizontalScale:
        """Return the ``MATH1:HORizontal:SCAle`` command.

        Description:
            - This command specifies the math horizontal display scale for FFT or for dual math
              waveforms that have source waveforms that are reference waveforms. The horizontal
              scale of a dual math waveform with a channel source waveform is set through the
              command.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:HORizontal:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal:SCAle?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:HORizontal:SCAle value``
              command.

        SCPI Syntax:
            ```
            - MATH1:HORizontal:SCAle <NR3>
            - MATH1:HORizontal:SCAle?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the math horizontal scale, in
              seconds.
        """
        return self._scale

    @property
    def units(self) -> Math1HorizontalUnits:
        """Return the ``MATH1:HORizontal:UNIts`` command.

        Description:
            - Returns the math waveform horizontal measurement unit value.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:HORizontal:UNIts?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal:UNIts?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MATH1:HORizontal:UNIts?
            ```
        """
        return self._units


class Math1Define(SCPICmdWrite, SCPICmdRead):
    r"""The ``MATH1:DEFine`` command.

    Description:
        - This command specifies the current math function as a text string. You must also specify
          the type (dual waveform, FFT, advanced math, or spectrum math) using ``MATH1:TYPE Dual``
          math expressions are strings of the form <wfm> <operation> <wfm>, where the <wfm>s are any
          combination of live analog or reference waveforms in the time domain display, the
          <operation> is any of +, -, \* or /, and the type has been set to DUAL. FFT math
          expressions are strings of the form FFT(<wfm>), where <wfm> is any live analog or
          reference waveforms in the time domain display, and the type has been set to FFT. Advanced
          math expressions extend beyond dual math expressions, incorporating combinations of
          advanced math functions, measurements, and operators. Spectrum math (MDO models)
          expressions are strings of the form <wfm><operation><wfm>, where the waveforms are any
          combination of live RF or reference traces in the frequency domain display, the
          <operation> is either + or -, and type has been set to SPECTRUM.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:DEFine?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:DEFine?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:DEFine value`` command.

    SCPI Syntax:
        ```
        - MATH1:DEFine <QString>
        - MATH1:DEFine?
        ```

    Info:
        - ``<QString>`` quoted string argument is the mathematical expression that defines the
          waveform.
        - ``:MATH:DEFINE 'FFT(CH1)'``
        - ``Automatic measurements of waveforms``
        - ``:MATH:DEFINE 'AMPlitude(CH1)'``
        - ``:MATH:DEFINE 'AREA(CH1)'``
        - ``:MATH:DEFINE 'BURST(CH1)'``
        - ``:MATH:DEFINE 'CAREA(CH1)'``
        - ``:MATH:DEFINE 'CMEAN(CH1)'``
        - ``:MATH:DEFINE 'CRMS(CH1)'``
        - ``:MATH:DEFINE 'DELAY(CH1,CH2)'``
        - ``:MATH:DEFINE 'FALL(CH1)'``
        - ``:MATH:DEFINE 'FREQUENCY(CH1)'``
        - ``:MATH:DEFINE 'HIGH(CH1)*CH2'``
        - ``:MATH:DEFINE 'LOW(CH1)'``
        - ``:MATH:DEFINE 'MAXIMUM(CH1)'``
        - ``:MATH:DEFINE 'MEAN(CH1)'``
        - ``:MATH:DEFINE 'MINIMUM(CH1)'``
        - ``:MATH:DEFINE 'NDUTY(CH1)'``
        - ``:MATH:DEFINE 'NOVERSHOOT(CH1)'``
        - ``:MATH:DEFINE 'NWIDTH(CH1)'``
        - ``:MATH:DEFINE 'PDUTY(CH1)'``
        - ``:MATH:DEFINE 'PERIOD(CH1)'``
        - ``:MATH:DEFINE 'PHASE(CH1,CH2)'``
        - ``:MATH:DEFINE 'PK2PK(CH1)'``
        - ``:MATH:DEFINE 'POVERSHOOT(CH1)'``
        - ``:MATH:DEFINE 'PWIDTH(CH1)'``
        - ``:MATH:DEFINE 'RISE(CH1)'``
        - ``:MATH:DEFINE 'RMS(CH1)'``
        - ``:MATH:DEFINE 'SINE(CH1)'``
        - ``Trigonometric operations on expressions``
        - ``:MATH:DEFINE 'ABS(CH2-CH1)'``
        - ``:MATH:DEFINE 'COSine(CH1)'``
        - ``:MATH:DEFINE 'CH1 * DEG(VAR1)'``
        - ``:MATH:DEFINE 'DIFF(CH1)'``
        - ``:MATH:DEFINE 'DIFF(ABS(CH1))'``
        - ``:MATH:DEFINE 'EXP(DIFF(CH1))'``
        - ``:MATH:DEFINE 'FFT(CH1)'``
        - ``:MATH:DEFINE 'FFT(CH1+CH2)'``
        - ``:MATH:DEFINE 'FFT(CH1+ INT(CH2))'``
        - ``:MATH:DEFINE 'CH1 + FFT(CH2)'`` Not acceptable.
        - ``:MATH:DEFINE 'INTG(CH1)'``
        - ``:MATH:DEFINE 'INTG(CH1+CH2)'``
        - ``:MATH:DEFINE 'LOG(CH1)'``
        - ``:MATH:DEFINE 'RAD(PHASE(CH1,CH2))'``
        - ``:MATH:DEFINE 'SQRT(SINE(CH1))'``
        - ``:MATH:DEFINE 'SINE(CH1+CH2)'``
        - ``:MATH:DEFINE 'TAN(CH1)'``
        - ``:MATH:DEFINE 'TRE(PERIOD(CH1))'``
        - ``User-defined variables``
        - ``:MATH:DEFINE 'VAR1+CH1'``
        - ``Relational and logical operators``
        - ``:MATH:DEFINE '(CH1 < CH2)'``
        - ``:MATH:DEFINE '(CH1 > CH2)'``
        - ``:MATH:DEFINE '(CH1 <= CH2)'``
        - ``:MATH:DEFINE '(CH1 >= CH2)'``
        - ``:MATH:DEFINE '(CH1 != CH2)'``
        - ``:MATH:DEFINE 'CH1==CH2'``
        - ``:MATH:DEFINE '(CH1 != CH2)``
        - ``(CH3 == CH4)'``
        - ``:MATH:DEFINE '(CH1 != CH2) && (CH3 == CH4)'``
        - ``:MATH:DEFINE '(CH2-CH1) * !(CH1 >= CH2)'``
    """

    _WRAP_ARG_WITH_QUOTES = True


class Math1Autoscale(SCPICmdWrite, SCPICmdRead):
    """The ``MATH1:AUTOSCale`` command.

    Description:
        - Sets (or queries) the state of automatic vertical scaling of the math waveform. The
          default state is ON (1).

    Usage:
        - Using the ``.query()`` method will send the ``MATH1:AUTOSCale?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1:AUTOSCale?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATH1:AUTOSCale value`` command.

    SCPI Syntax:
        ```
        - MATH1:AUTOSCale {ON|OFF|1|0}
        - MATH1:AUTOSCale?
        ```

    Info:
        - ``1`` or ON enables automatic vertical scaling of the math waveform when the RF reference
          level falls below -25.0 dBm. Reference levels greater than or equal to -25.0 dBm cause the
          probe to go into BYPass mode. The reference level is set/queried using the command
          ``RF:REFLevel``.
        - ``0`` or OFF disables math autoscaling.
    """


class Math1(SCPICmdRead):
    """The ``MATH1`` command.

    Description:
        - Returns the definition of the math waveform. The returned data depends on the setting of
          the ``MATH1:TYPE`` command.

    Usage:
        - Using the ``.query()`` method will send the ``MATH1?`` query.
        - Using the ``.verify(value)`` method will send the ``MATH1?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MATH1?
        ```

    Properties:
        - ``.autoscale``: The ``MATH1:AUTOSCale`` command.
        - ``.define``: The ``MATH1:DEFine`` command.
        - ``.horizontal``: The ``MATH1:HORizontal`` command tree.
        - ``.label``: The ``MATH1:LABel`` command.
        - ``.spectral``: The ``MATH1:SPECTral`` command tree.
        - ``.type``: The ``MATH1:TYPe`` command.
        - ``.vertical``: The ``MATH1:VERTical`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "MATH1") -> None:
        super().__init__(device, cmd_syntax)
        self._autoscale = Math1Autoscale(device, f"{self._cmd_syntax}:AUTOSCale")
        self._define = Math1Define(device, f"{self._cmd_syntax}:DEFine")
        self._horizontal = Math1Horizontal(device, f"{self._cmd_syntax}:HORizontal")
        self._label = Math1Label(device, f"{self._cmd_syntax}:LABel")
        self._spectral = Math1Spectral(device, f"{self._cmd_syntax}:SPECTral")
        self._type = Math1Type(device, f"{self._cmd_syntax}:TYPe")
        self._vertical = Math1Vertical(device, f"{self._cmd_syntax}:VERTical")

    @property
    def autoscale(self) -> Math1Autoscale:
        """Return the ``MATH1:AUTOSCale`` command.

        Description:
            - Sets (or queries) the state of automatic vertical scaling of the math waveform. The
              default state is ON (1).

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:AUTOSCale?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:AUTOSCale?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:AUTOSCale value`` command.

        SCPI Syntax:
            ```
            - MATH1:AUTOSCale {ON|OFF|1|0}
            - MATH1:AUTOSCale?
            ```

        Info:
            - ``1`` or ON enables automatic vertical scaling of the math waveform when the RF
              reference level falls below -25.0 dBm. Reference levels greater than or equal to -25.0
              dBm cause the probe to go into BYPass mode. The reference level is set/queried using
              the command ``RF:REFLevel``.
            - ``0`` or OFF disables math autoscaling.
        """
        return self._autoscale

    @property
    def define(self) -> Math1Define:
        r"""Return the ``MATH1:DEFine`` command.

        Description:
            - This command specifies the current math function as a text string. You must also
              specify the type (dual waveform, FFT, advanced math, or spectrum math) using
              ``MATH1:TYPE Dual`` math expressions are strings of the form <wfm> <operation> <wfm>,
              where the <wfm>s are any combination of live analog or reference waveforms in the time
              domain display, the <operation> is any of +, -, \* or /, and the type has been set to
              DUAL. FFT math expressions are strings of the form FFT(<wfm>), where <wfm> is any live
              analog or reference waveforms in the time domain display, and the type has been set to
              FFT. Advanced math expressions extend beyond dual math expressions, incorporating
              combinations of advanced math functions, measurements, and operators. Spectrum math
              (MDO models) expressions are strings of the form <wfm><operation><wfm>, where the
              waveforms are any combination of live RF or reference traces in the frequency domain
              display, the <operation> is either + or -, and type has been set to SPECTRUM.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:DEFine?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:DEFine?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:DEFine value`` command.

        SCPI Syntax:
            ```
            - MATH1:DEFine <QString>
            - MATH1:DEFine?
            ```

        Info:
            - ``<QString>`` quoted string argument is the mathematical expression that defines the
              waveform.
            - ``:MATH:DEFINE 'FFT(CH1)'``
            - ``Automatic measurements of waveforms``
            - ``:MATH:DEFINE 'AMPlitude(CH1)'``
            - ``:MATH:DEFINE 'AREA(CH1)'``
            - ``:MATH:DEFINE 'BURST(CH1)'``
            - ``:MATH:DEFINE 'CAREA(CH1)'``
            - ``:MATH:DEFINE 'CMEAN(CH1)'``
            - ``:MATH:DEFINE 'CRMS(CH1)'``
            - ``:MATH:DEFINE 'DELAY(CH1,CH2)'``
            - ``:MATH:DEFINE 'FALL(CH1)'``
            - ``:MATH:DEFINE 'FREQUENCY(CH1)'``
            - ``:MATH:DEFINE 'HIGH(CH1)*CH2'``
            - ``:MATH:DEFINE 'LOW(CH1)'``
            - ``:MATH:DEFINE 'MAXIMUM(CH1)'``
            - ``:MATH:DEFINE 'MEAN(CH1)'``
            - ``:MATH:DEFINE 'MINIMUM(CH1)'``
            - ``:MATH:DEFINE 'NDUTY(CH1)'``
            - ``:MATH:DEFINE 'NOVERSHOOT(CH1)'``
            - ``:MATH:DEFINE 'NWIDTH(CH1)'``
            - ``:MATH:DEFINE 'PDUTY(CH1)'``
            - ``:MATH:DEFINE 'PERIOD(CH1)'``
            - ``:MATH:DEFINE 'PHASE(CH1,CH2)'``
            - ``:MATH:DEFINE 'PK2PK(CH1)'``
            - ``:MATH:DEFINE 'POVERSHOOT(CH1)'``
            - ``:MATH:DEFINE 'PWIDTH(CH1)'``
            - ``:MATH:DEFINE 'RISE(CH1)'``
            - ``:MATH:DEFINE 'RMS(CH1)'``
            - ``:MATH:DEFINE 'SINE(CH1)'``
            - ``Trigonometric operations on expressions``
            - ``:MATH:DEFINE 'ABS(CH2-CH1)'``
            - ``:MATH:DEFINE 'COSine(CH1)'``
            - ``:MATH:DEFINE 'CH1 * DEG(VAR1)'``
            - ``:MATH:DEFINE 'DIFF(CH1)'``
            - ``:MATH:DEFINE 'DIFF(ABS(CH1))'``
            - ``:MATH:DEFINE 'EXP(DIFF(CH1))'``
            - ``:MATH:DEFINE 'FFT(CH1)'``
            - ``:MATH:DEFINE 'FFT(CH1+CH2)'``
            - ``:MATH:DEFINE 'FFT(CH1+ INT(CH2))'``
            - ``:MATH:DEFINE 'CH1 + FFT(CH2)'`` Not acceptable.
            - ``:MATH:DEFINE 'INTG(CH1)'``
            - ``:MATH:DEFINE 'INTG(CH1+CH2)'``
            - ``:MATH:DEFINE 'LOG(CH1)'``
            - ``:MATH:DEFINE 'RAD(PHASE(CH1,CH2))'``
            - ``:MATH:DEFINE 'SQRT(SINE(CH1))'``
            - ``:MATH:DEFINE 'SINE(CH1+CH2)'``
            - ``:MATH:DEFINE 'TAN(CH1)'``
            - ``:MATH:DEFINE 'TRE(PERIOD(CH1))'``
            - ``User-defined variables``
            - ``:MATH:DEFINE 'VAR1+CH1'``
            - ``Relational and logical operators``
            - ``:MATH:DEFINE '(CH1 < CH2)'``
            - ``:MATH:DEFINE '(CH1 > CH2)'``
            - ``:MATH:DEFINE '(CH1 <= CH2)'``
            - ``:MATH:DEFINE '(CH1 >= CH2)'``
            - ``:MATH:DEFINE '(CH1 != CH2)'``
            - ``:MATH:DEFINE 'CH1==CH2'``
            - ``:MATH:DEFINE '(CH1 != CH2)``
            - ``(CH3 == CH4)'``
            - ``:MATH:DEFINE '(CH1 != CH2) && (CH3 == CH4)'``
            - ``:MATH:DEFINE '(CH2-CH1) * !(CH1 >= CH2)'``
        """
        return self._define

    @property
    def horizontal(self) -> Math1Horizontal:
        """Return the ``MATH1:HORizontal`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:HORizontal?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:HORizontal?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.position``: The ``MATH1:HORizontal:POSition`` command.
            - ``.scale``: The ``MATH1:HORizontal:SCAle`` command.
            - ``.units``: The ``MATH1:HORizontal:UNIts`` command.
        """
        return self._horizontal

    @property
    def label(self) -> Math1Label:
        """Return the ``MATH1:LABel`` command.

        Description:
            - Sets or queries the waveform label for the math waveform.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:LABel?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:LABel value`` command.

        SCPI Syntax:
            ```
            - MATH1:LABel <QString>
            - MATH1:LABel?
            ```

        Info:
            - ``<QString>`` is the quoted string used as the label for the math waveform.
        """
        return self._label

    @property
    def spectral(self) -> Math1Spectral:
        """Return the ``MATH1:SPECTral`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:SPECTral?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:SPECTral?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.mag``: The ``MATH1:SPECTral:MAG`` command.
            - ``.window``: The ``MATH1:SPECTral:WINdow`` command.
        """
        return self._spectral

    @property
    def type(self) -> Math1Type:
        """Return the ``MATH1:TYPe`` command.

        Description:
            - This command specifies the math type ( DUAL, FFT, ADVanced or SPECTRUM ). This command
              is used along with ``MATH1:DEFINE``, which specifies the current mathematical
              expression as a text string which defines the waveform. You must specify the math type
              before defining the math expression. For a list of math expressions, see
              ``MATH1:DEFINE``.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:TYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:TYPe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATH1:TYPe value`` command.

        SCPI Syntax:
            ```
            - MATH1:TYPe {DUAL|FFT|ADVanced|SPECTRUM}
            - MATH1:TYPe?
            ```

        Info:
            - ``DUAL`` sets the type to dual waveform math, which can use any combination of live
              analog or reference waveforms in the time domain display.
            - ``FFT`` sets the type to FFT math, which can use any live analog or reference waveform
              in the time domain.
            - ``ADVanced`` sets the type to advanced math.
            - ``SPECTRUM`` sets the type to spectrum, which can use any combination of live RF or
              reference traces in the frequency domain. This command requires option 3-SA3.
        """
        return self._type

    @property
    def vertical(self) -> Math1Vertical:
        """Return the ``MATH1:VERTical`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MATH1:VERTical?`` query.
            - Using the ``.verify(value)`` method will send the ``MATH1:VERTical?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.position``: The ``MATH1:VERTical:POSition`` command.
            - ``.scale``: The ``MATH1:VERTical:SCAle`` command.
            - ``.units``: The ``MATH1:VERTical:UNIts`` command.
        """
        return self._vertical



"""The searchtable commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SEARCHTABle:ADDNew <qstring>
    - SEARCHTABle:DELete <qstring>
    - SEARCHTABle:list?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SearchtableList(SCPICmdRead):
    """The ``SEARCHTABle:list`` command.

    Description:
        - This command queries all search event table in an Option 5-WIN (Microsoft Windows 10 OS)
          TekExpress compliance testing application.

    Usage:
        - Using the ``.query()`` method will send the ``SEARCHTABle:list?`` query.
        - Using the ``.verify(value)`` method will send the ``SEARCHTABle:list?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SEARCHTABle:list?
        ```
    """


class SearchtableDelete(SCPICmdWrite):
    """The ``SEARCHTABle:DELete`` command.

    Description:
        - This command deletes a search event table in an Option 5-WIN (Microsoft Windows 10 OS)
          TekExpress compliance testing application.

    Usage:
        - Using the ``.write(value)`` method will send the ``SEARCHTABle:DELete value`` command.

    SCPI Syntax:
        ```
        - SEARCHTABle:DELete <qstring>
        ```

    Info:
        - ``<qstring>`` contains the name of the search table. 'TABLE' should be the argument prefix
          followed by a non-zero integer.
    """


class SearchtableAddnew(SCPICmdWrite):
    """The ``SEARCHTABle:ADDNew`` command.

    Description:
        - This command adds a new search event table in an Option 5-WIN (Microsoft Windows 10 OS)
          TekExpress compliance testing application.

    Usage:
        - Using the ``.write(value)`` method will send the ``SEARCHTABle:ADDNew value`` command.

    SCPI Syntax:
        ```
        - SEARCHTABle:ADDNew <qstring>
        ```

    Info:
        - ``<qstring>`` contains the name of the search table. 'TABLE' should be the argument prefix
          followed by a non-zero integer.
    """


class Searchtable(SCPICmdRead):
    """The ``SEARCHTABle`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SEARCHTABle?`` query.
        - Using the ``.verify(value)`` method will send the ``SEARCHTABle?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.addnew``: The ``SEARCHTABle:ADDNew`` command.
        - ``.delete``: The ``SEARCHTABle:DELete`` command.
        - ``.list``: The ``SEARCHTABle:list`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "SEARCHTABle"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._addnew = SearchtableAddnew(device, f"{self._cmd_syntax}:ADDNew")
        self._delete = SearchtableDelete(device, f"{self._cmd_syntax}:DELete")
        self._list = SearchtableList(device, f"{self._cmd_syntax}:list")

    @property
    def addnew(self) -> SearchtableAddnew:
        """Return the ``SEARCHTABle:ADDNew`` command.

        Description:
            - This command adds a new search event table in an Option 5-WIN (Microsoft Windows 10
              OS) TekExpress compliance testing application.

        Usage:
            - Using the ``.write(value)`` method will send the ``SEARCHTABle:ADDNew value`` command.

        SCPI Syntax:
            ```
            - SEARCHTABle:ADDNew <qstring>
            ```

        Info:
            - ``<qstring>`` contains the name of the search table. 'TABLE' should be the argument
              prefix followed by a non-zero integer.
        """
        return self._addnew

    @property
    def delete(self) -> SearchtableDelete:
        """Return the ``SEARCHTABle:DELete`` command.

        Description:
            - This command deletes a search event table in an Option 5-WIN (Microsoft Windows 10 OS)
              TekExpress compliance testing application.

        Usage:
            - Using the ``.write(value)`` method will send the ``SEARCHTABle:DELete value`` command.

        SCPI Syntax:
            ```
            - SEARCHTABle:DELete <qstring>
            ```

        Info:
            - ``<qstring>`` contains the name of the search table. 'TABLE' should be the argument
              prefix followed by a non-zero integer.
        """
        return self._delete

    @property
    def list(self) -> SearchtableList:
        """Return the ``SEARCHTABle:list`` command.

        Description:
            - This command queries all search event table in an Option 5-WIN (Microsoft Windows 10
              OS) TekExpress compliance testing application.

        Usage:
            - Using the ``.query()`` method will send the ``SEARCHTABle:list?`` query.
            - Using the ``.verify(value)`` method will send the ``SEARCHTABle:list?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SEARCHTABle:list?
            ```
        """
        return self._list
